@@ -31,13 +31,16 @@
 typedef struct
 {
     GSettings *desktop;
+    GSettings *settings;
     gchar *monospace_font;
 } GlApplicationPrivate;
 
 G_DEFINE_TYPE_WITH_PRIVATE (GlApplication, gl_application, GTK_TYPE_APPLICATION)
 
 static const gchar DESKTOP_SCHEMA[] = "org.gnome.desktop.interface";
+static const gchar SETTINGS_SCHEMA[] = "org.gnome.Logs";
 static const gchar DESKTOP_MONOSPACE_FONT_NAME[] = "monospace-font-name";
+static const gchar SORT_ORDER[] = "sort-order";
 
 static void
 on_new_window (GSimpleAction *action,
@@ -117,6 +120,25 @@ on_quit (GSimpleAction *action,
     g_application_quit (application);
 }
 
+static void
+on_sort_order_changed (GSettings *settings,
+                       const gchar *key,
+                       GlApplication *application)
+{
+    GList *window;
+    gint sort_order;
+
+    window = gtk_application_get_windows (GTK_APPLICATION (application));
+    sort_order = g_settings_get_enum (settings, SORT_ORDER);
+
+    /* refresh the event view list for every logs window */
+    while (window != NULL)
+    {
+        gl_window_set_sort_order (window->data, sort_order);
+        window = g_list_next (window);
+    }
+}
+
 static void
 on_monospace_font_name_changed (GSettings *settings,
                                 const gchar *key,
@@ -233,6 +255,7 @@ gl_application_finalize (GObject *object)
     priv = gl_application_get_instance_private (application);
 
     g_clear_object (&priv->desktop);
+    g_clear_object (&priv->settings);
     g_clear_pointer (&priv->monospace_font, g_free);
 }
 
@@ -241,6 +264,7 @@ gl_application_init (GlApplication *application)
 {
     GlApplicationPrivate *priv;
     gchar *changed_font;
+    GAction *action;
 
     priv = gl_application_get_instance_private (application);
 
@@ -254,6 +278,15 @@ gl_application_init (GlApplication *application)
                       G_CALLBACK (on_monospace_font_name_changed),
                       priv);
 
+    priv->settings = g_settings_new (SETTINGS_SCHEMA);
+    action = g_settings_create_action (priv->settings, SORT_ORDER);
+    g_action_map_add_action (G_ACTION_MAP (application), action);
+
+    g_signal_connect (priv->settings, "changed::sort-order",
+                      G_CALLBACK (on_sort_order_changed),
+                      application);
+
+    g_object_unref (action);
     g_free (changed_font);
 }
 
