@@ -23,18 +23,32 @@
 #include <stdlib.h>
 #include <systemd/sd-journal.h>
 
+struct _GlJournalEntry
+{
+  GObject parent_instance;
+
+  guint64 timestamp;
+  gchar *cursor;
+  gchar *message;
+  gchar *comm;
+  gchar *kernel_device;
+  gchar *audit_session;
+  gchar *catalog;
+  guint priority;
+};
+
+G_DEFINE_TYPE (GlJournalEntry, gl_journal_entry, G_TYPE_OBJECT);
+
 typedef struct
 {
     sd_journal *journal;
     gint fd;
     guint source_id;
+    gchar **mandatory_fields;
 } GlJournalPrivate;
 
 G_DEFINE_TYPE_WITH_PRIVATE (GlJournal, gl_journal, G_TYPE_OBJECT)
 
-G_DEFINE_BOXED_TYPE (GlJournalResult, gl_journal_result, gl_journal_result_ref,
-                     gl_journal_result_unref)
-
 GQuark
 gl_journal_error_quark (void)
 {
@@ -79,6 +93,7 @@ gl_journal_finalize (GObject *object)
 
     g_source_remove (priv->source_id);
     g_clear_pointer (&priv->journal, sd_journal_close);
+    g_clear_pointer (&priv->mandatory_fields, g_strfreev);
 }
 
 static void
@@ -192,11 +207,11 @@ gl_journal_get_data (GlJournal *self,
     return g_strndup ((const gchar *)data + prefix_len, length - prefix_len);
 }
 
-static GlJournalResult *
-_gl_journal_query_result (GlJournal *self)
+static GlJournalEntry *
+_gl_journal_query_entry (GlJournal *self)
 {
     GlJournalPrivate *priv;
-    GlJournalResult *result;
+    GlJournalEntry *entry;
     gint ret;
     sd_journal *journal;
     GError *error = NULL;
@@ -205,11 +220,9 @@ _gl_journal_query_result (GlJournal *self)
     priv = gl_journal_get_instance_private (self);
     journal = priv->journal;
 
-    result = g_slice_new (GlJournalResult);
+    entry = g_object_new (GL_TYPE_JOURNAL_ENTRY, NULL);
 
-    result->ref_count = 1;
-
-    ret = sd_journal_get_realtime_usec (journal, &result->timestamp);
+    ret = sd_journal_get_realtime_usec (journal, &entry->timestamp);
 
     if (ret < 0)
     {
@@ -218,7 +231,7 @@ _gl_journal_query_result (GlJournal *self)
         goto out;
     }
 
-    ret = sd_journal_get_cursor (journal, &result->cursor);
+    ret = sd_journal_get_cursor (journal, &entry->cursor);
 
     if (ret < 0)
     {
@@ -227,13 +240,13 @@ _gl_journal_query_result (GlJournal *self)
         goto out;
     }
 
-    ret = sd_journal_test_cursor (journal, result->cursor);
+    ret = sd_journal_test_cursor (journal, entry->cursor);
 
     if (ret < 0)
     {
         g_warning ("Error testing cursor string: %s", g_strerror (-ret));
-        free (result->cursor);
-        result->cursor = NULL;
+        free (entry->cursor);
+        entry->cursor = NULL;
         goto out;
     }
     else if (ret == 0)
@@ -243,29 +256,29 @@ _gl_journal_query_result (GlJournal *self)
          * later on. */
     }
 
-    ret = sd_journal_get_catalog (journal, &result->catalog);
+    ret = sd_journal_get_catalog (journal, &entry->catalog);
 
     if (ret == -ENOENT)
     {
         g_debug ("No message for this log entry was found in the catalog");
-        result->catalog = NULL;
+        entry->catalog = NULL;
     }
     else if (ret < 0)
     {
         g_warning ("Error while getting message from catalog: %s",
                    g_strerror (-ret));
-        free (result->cursor);
+        free (entry->cursor);
         goto out;
     }
 
-    result->message = gl_journal_get_data (self, "MESSAGE", NULL);
+    entry->message = gl_journal_get_data (self, "MESSAGE", NULL);
 
     if (error != NULL)
     {
         g_warning ("%s", error->message);
         g_clear_error (&error);
-        free (result->cursor);
-        free (result->catalog);
+        free (entry->cursor);
+        free (entry->catalog);
         goto out;
     }
 
@@ -275,16 +288,16 @@ _gl_journal_query_result (GlJournal *self)
     {
         g_warning ("%s", error->message);
         g_clear_error (&error);
-        free (result->cursor);
-        free (result->catalog);
-        g_free (result->message);
+        free (entry->cursor);
+        free (entry->catalog);
+        g_free (entry->message);
         goto out;
     }
 
-    result->priority = priority ? atoi (priority) : LOG_INFO;
+    entry->priority = priority ? atoi (priority) : LOG_INFO;
     g_free (priority);
 
-    result->comm = gl_journal_get_data (self, "_COMM", &error);
+    entry->comm = gl_journal_get_data (self, "_COMM", &error);
 
     if (error != NULL)
     {
@@ -292,7 +305,7 @@ _gl_journal_query_result (GlJournal *self)
         g_clear_error (&error);
     }
 
-    result->kernel_device = gl_journal_get_data (self, "_KERNEL_DEVICE", NULL);
+    entry->kernel_device = gl_journal_get_data (self, "_KERNEL_DEVICE", NULL);
 
     if (error != NULL)
     {
@@ -300,318 +313,169 @@ _gl_journal_query_result (GlJournal *self)
         g_clear_error (&error);
     }
 
-    result->audit_session = gl_journal_get_data (self, "_AUDIT_SESSION", NULL);
+    entry->audit_session = gl_journal_get_data (self, "_AUDIT_SESSION", NULL);
 
-    return result;
+    return entry;
 
 out:
-    g_slice_free (GlJournalResult, result);
+    g_object_unref (entry);
 
     return NULL;
 }
 
-static GlJournalQuery *
-gl_journal_query_copy (const GlJournalQuery *source)
+/**
+ * gl_journal_query_match:
+ * @query: a @query to match against the current log entry
+ *
+ * systemd's matching doesn't allow checking for the existance of
+ * fields. This function checks if fields in @query that don't have a
+ * value (and thus no '=' delimiter) are present at all in the current
+ * log entry.
+ *
+ * Returns: %TRUE if the current log entry contains all fields in @query
+ */
+static gboolean
+gl_journal_query_match (sd_journal          *journal,
+                        const gchar * const *query)
 {
-    GlJournalQuery *result;
-    guint n_matches = 0;
-    gchar **matches;
-    guint i;
+  gint i;
 
-    result = g_slice_new (GlJournalQuery);
+  for (i = 0; query[i]; i++)
+  {
+      int r;
+      const void *data;
+      size_t len;
 
-    if (source->matches)
+      /* don't check fields that match on a value */
+      if (strchr (query[i], '='))
+          continue;
+
+      r = sd_journal_get_data (journal, query[i], &data, &len);
+
+      if (r == -ENOENT) /* field doesn't exist */
+          return FALSE;
+
+      else if (r < 0)
+          g_warning ("Failed to read log entry: %s", g_strerror (-r));
+  }
+
+  return TRUE;
+}
+
+static gchar *
+create_boot_id_match_string (void)
+{
+    sd_id128_t boot_id;
+    gchar boot_string[33];
+    int r;
+
+    r = sd_id128_get_boot (&boot_id);
+    if (r < 0)
     {
-        n_matches = g_strv_length (source->matches);
+        g_warning ("Error getting boot ID of running kernel: %s", g_strerror (-r));
+        return NULL;
     }
 
-    /* Remember the trailing NULL. */
-    matches = g_new (gchar *, n_matches + 1);
-
-    for (i = 0; i < n_matches; i++)
-    {
-        matches[i] = g_strdup (source->matches[i]);
-    }
-
-    matches[n_matches] = NULL;
-
-    result->matches = matches;
-    result->n_results = source->n_results;
-
-    return result;
-}
-
-static void
-gl_journal_query_free (GlJournalQuery *query)
-{
-    g_strfreev (query->matches);
-    g_slice_free (GlJournalQuery, query);
-}
-
-static void
-gl_journal_query_async_thread (GTask *task,
-                               gpointer source_object,
-                               gpointer task_data,
-                               GCancellable *cancellable)
-{
-    GList *results;
-
-    /* TODO: gl_journal_query() should accept a cancellable. */
-    results = gl_journal_query (GL_JOURNAL (source_object),
-                                (GlJournalQuery *)task_data);
-    g_task_return_pointer (task, results,
-                           (GDestroyNotify)gl_journal_results_free);
-    /* TODO: Return error if necessary (if cancelled?). */
+    sd_id128_to_string (boot_id, boot_string);
+    return g_strconcat ("_BOOT_ID=", boot_string, NULL);
 }
 
+/**
+ * gl_journal_set_matches:
+ * @journal: a #GlJournal
+ * @matches: new matches to set
+ *
+ * Sets @matches on @journal. Will reset the cursor position to the
+ * beginning.
+ */
 void
-gl_journal_query_async (GlJournal *self,
-                        const GlJournalQuery *query,
-                        GCancellable *cancellable,
-                        GAsyncReadyCallback callback,
-                        gpointer user_data)
+gl_journal_set_matches (GlJournal           *journal,
+                        const gchar * const *matches)
 {
-    GTask *task;
-    GlJournalQuery *data;
+    GlJournalPrivate *priv = gl_journal_get_instance_private (journal);
+    GPtrArray *mandatory_fields;
+    int r;
+    gint i;
+    gboolean has_boot_id;
 
-    data = gl_journal_query_copy (query);
+    g_return_if_fail (matches != NULL);
 
-    task = g_task_new (self, cancellable, callback, user_data);
-    g_task_set_task_data (task, data, (GDestroyNotify)gl_journal_query_free);
-    g_task_run_in_thread (task, gl_journal_query_async_thread);
-    g_object_unref (task);
-}
+    if (priv->mandatory_fields)
+      g_clear_pointer (&priv->mandatory_fields, g_strfreev);
 
-GList *
-gl_journal_query_finish (GlJournal *self,
-                         GAsyncResult *res,
-                         GError **error)
-{
-    g_return_val_if_fail (g_task_is_valid (res, self), NULL);
+    sd_journal_flush_matches (priv->journal);
 
-    return g_task_propagate_pointer (G_TASK (res), error);
-}
-
-GList *
-gl_journal_query (GlJournal *self, const GlJournalQuery *query)
-{
-    GlJournalPrivate *priv;
-    sd_journal *journal;
-    gsize i;
-    gint ret;
-    GList *results = NULL;
-
-    g_return_val_if_fail (GL_JOURNAL (self), NULL);
-    g_return_val_if_fail (query != NULL, NULL);
-
-    priv = gl_journal_get_instance_private (self);
-    journal = priv->journal;
-
-    if (query->matches)
+    mandatory_fields = g_ptr_array_new ();
+    for (i = 0; matches[i]; i++)
     {
-        const gchar *match;
-
-        for (i = 0, match = query->matches[i]; match;
-             match = query->matches[++i])
+        /* matches without a value should only check for existence.
+         * systemd doesn't support that, so let's remember them to
+         * filter out later.
+         */
+        if (strchr (matches[i], '=') == NULL)
         {
-            ret = sd_journal_add_match (journal, match, 0);
-
-            if (ret < 0)
-            {
-                g_warning ("Error adding match '%s': %s", match,
-                           g_strerror (-ret));
-            }
-        }
-    }
-
-    if (query->n_results == -1)
-    {
-        /* Take events from this boot only. */
-        sd_id128_t boot_id;
-        gchar boot_string[33];
-        gchar *match_string;
-
-        ret = sd_id128_get_boot (&boot_id);
-
-        if (ret < 0)
-        {
-            g_warning ("Error getting boot ID of running kernel: %s",
-                       g_strerror (-ret));
-        }
-
-        sd_id128_to_string (boot_id, boot_string);
-
-        match_string = g_strconcat ("_BOOT_ID=", boot_string, NULL);
-
-        ret = sd_journal_add_match (journal, match_string, 0);
-
-        if (ret < 0)
-        {
-            g_warning ("Error adding match '%s': %s", match_string,
-                       g_strerror (-ret));
-        }
-
-        g_free (match_string);
-
-        ret = sd_journal_seek_head (journal);
-
-        if (ret < 0)
-        {
-            g_warning ("Error seeking to start of systemd journal: %s",
-                       g_strerror (-ret));
-        }
-
-        do
-        {
-            GlJournalResult *result;
-
-            ret = sd_journal_next (journal);
-
-            if (ret < 0)
-            {
-                g_warning ("Error setting cursor to next position in systemd journal: %s",
-                           g_strerror (-ret));
-                break;
-            }
-            else if (ret == 0)
-            {
-                g_debug ("End of systemd journal reached");
-                break;
-            }
-
-            result = _gl_journal_query_result (self);
-
-            results = g_list_prepend (results, result);
-        } while (TRUE);
-    }
-    else
-    {
-        /* Take the given number of events from the end of the journal
-         * backwards. */
-        ret = sd_journal_seek_tail (journal);
-
-        if (ret < 0)
-        {
-            g_warning ("Error seeking to end of systemd journal: %s",
-                       g_strerror (-ret));
-        }
-
-        for (i = 0; i < query->n_results; i++)
-        {
-            GlJournalResult *result;
-
-            ret = sd_journal_previous (journal);
-
-            if (ret < 0)
-            {
-                g_warning ("Error setting cursor to end of systemd journal: %s",
-                           g_strerror (-ret));
-                break;
-            }
-            else if (ret == 0)
-            {
-                g_debug ("End of systemd journal reached");
-                break;
-            }
-
-            result = _gl_journal_query_result (self);
-
-            results = g_list_prepend (results, result);
+            g_ptr_array_add (mandatory_fields, g_strdup (matches[i]));
             continue;
         }
 
-        results = g_list_reverse (results);
+        if (g_str_has_prefix (matches[i], "_BOOT_ID="))
+          has_boot_id = TRUE;
+
+        r = sd_journal_add_match (priv->journal, matches[i], 0);
+        if (r < 0)
+        {
+            g_critical ("Failed to add match '%s': %s", matches[i], g_strerror (-r));
+            break;
+        }
     }
 
-    sd_journal_flush_matches (journal);
+    /* add sentinel */
+    g_ptr_array_add (mandatory_fields, NULL);
 
-    return results;
+    /* take events from this boot only, unless _BOOT_ID was in @matches */
+    if (!has_boot_id)
+    {
+        gchar *match;
+
+        match = create_boot_id_match_string ();
+        if (match)
+        {
+            r = sd_journal_add_match (priv->journal, match, 0);
+            if (r < 0)
+                g_warning ("Failed to add match '%s': %s", matches[i], g_strerror (-r));
+
+            g_free (match);
+        }
+    }
+
+    priv->mandatory_fields = (gchar **) g_ptr_array_free (mandatory_fields, FALSE);
+
+    r = sd_journal_seek_tail (priv->journal);
+    if (r < 0)
+        g_warning ("Error seeking to start of systemd journal: %s", g_strerror (-r));
 }
 
-GlJournalResult *
-gl_journal_query_cursor (GlJournal *self,
-                         const gchar *cursor)
+GlJournalEntry *
+gl_journal_previous (GlJournal *journal)
 {
-    GlJournalPrivate *priv;
-    sd_journal *journal;
-    gint ret;
-    GlJournalResult *result = NULL;
+    GlJournalPrivate *priv = gl_journal_get_instance_private (journal);
+    gint r;
 
-    g_return_val_if_fail (GL_JOURNAL (self), NULL);
-    g_return_val_if_fail (cursor != NULL, NULL);
-
-    priv = gl_journal_get_instance_private (self);
-    journal = priv->journal;
-
-    ret = sd_journal_seek_cursor (journal, cursor);
-
-    if (ret < 0)
+    r = sd_journal_previous (priv->journal);
+    if (r < 0)
     {
-        g_warning ("Error seeking to cursor position: %s", g_strerror (-ret));
-        goto out;
+        g_warning ("Failed to fetch previous log entry: %s", g_strerror (-r));
+        return NULL;
     }
 
-    ret = sd_journal_next (journal);
+    if (r == 0) /* end */
+        return NULL;
 
-    if (ret < 0)
-    {
-        g_warning ("Error positioning cursor in systemd journal: %s",
-                   g_strerror (-ret));
-    }
+    /* filter this one out because of a non-existent field */
+    if (!gl_journal_query_match (priv->journal, (const gchar * const *) priv->mandatory_fields))
+        return gl_journal_previous (journal);
 
-    ret = sd_journal_test_cursor (journal, cursor);
-
-    if (ret < 0)
-    {
-        g_warning ("Error testing cursor string: %s", g_strerror (-ret));
-        goto out;
-    }
-    else if (ret == 0)
-    {
-        g_warning ("Cursor string does not match journal entry");
-        goto out;
-    }
-
-    result = _gl_journal_query_result (self);
-
-out:
-    return result;
-}
-
-static void
-gl_journal_result_free (GlJournalResult *result,
-                        G_GNUC_UNUSED gpointer user_data)
-{
-    gl_journal_result_unref (result);
-}
-
-void
-gl_journal_results_free (GList *results)
-{
-    g_list_foreach (results, (GFunc)gl_journal_result_free, NULL);
-    g_list_free (results);
-}
-
-GlJournalResult *
-gl_journal_result_ref (GlJournalResult *result)
-{
-    g_atomic_int_inc (&result->ref_count);
-    return result;
-}
-
-void
-gl_journal_result_unref (GlJournalResult *result)
-{
-    if (g_atomic_int_dec_and_test (&result->ref_count))
-    {
-        free (result->cursor);
-        free (result->catalog);
-        g_free (result->message);
-        g_free (result->comm);
-        g_free (result->kernel_device);
-        g_free (result->audit_session);
-        g_slice_free (GlJournalResult, result);
-    }
+    return _gl_journal_query_entry (journal);
 }
 
 GlJournal *
@@ -619,3 +483,87 @@ gl_journal_new (void)
 {
     return g_object_new (GL_TYPE_JOURNAL, NULL);
 }
+
+static void
+gl_journal_entry_init (GlJournalEntry *entry)
+{
+}
+
+static void
+gl_journal_entry_finalize (GObject *object)
+{
+  GlJournalEntry *entry = GL_JOURNAL_ENTRY (object);
+
+  free (entry->cursor);
+  free (entry->catalog);
+  g_free (entry->message);
+  g_free (entry->comm);
+  g_free (entry->kernel_device);
+  g_free (entry->audit_session);
+
+  G_OBJECT_CLASS (gl_journal_entry_parent_class)->finalize (object);
+}
+
+static void
+gl_journal_entry_class_init (GlJournalEntryClass *class)
+{
+  GObjectClass *object_class = G_OBJECT_CLASS (class);
+
+  object_class->finalize = gl_journal_entry_finalize;
+}
+
+guint64
+gl_journal_entry_get_timestamp (GlJournalEntry *entry)
+{
+  g_return_val_if_fail (GL_IS_JOURNAL_ENTRY (entry), 0);
+
+  return entry->timestamp;
+}
+
+const gchar *
+gl_journal_entry_get_message (GlJournalEntry *entry)
+{
+  g_return_val_if_fail (GL_IS_JOURNAL_ENTRY (entry), NULL);
+
+  return entry->message;
+}
+
+const gchar *
+gl_journal_entry_get_command_line (GlJournalEntry *entry)
+{
+  g_return_val_if_fail (GL_IS_JOURNAL_ENTRY (entry), NULL);
+
+  return entry->comm;
+}
+
+const gchar *
+gl_journal_entry_get_kernel_device (GlJournalEntry *entry)
+{
+  g_return_val_if_fail (GL_IS_JOURNAL_ENTRY (entry), NULL);
+
+  return entry->kernel_device;
+}
+
+const gchar *
+gl_journal_entry_get_audit_session (GlJournalEntry *entry)
+{
+  g_return_val_if_fail (GL_IS_JOURNAL_ENTRY (entry), NULL);
+
+  return entry->audit_session;
+}
+
+const gchar *
+gl_journal_entry_get_catalog (GlJournalEntry *entry)
+{
+  g_return_val_if_fail (GL_IS_JOURNAL_ENTRY (entry), NULL);
+
+  return entry->catalog;
+}
+
+guint
+gl_journal_entry_get_priority (GlJournalEntry *entry)
+{
+  g_return_val_if_fail (GL_IS_JOURNAL_ENTRY (entry), 0);
+
+  return entry->priority;
+}
